/************************************************************

	Author:						Manfred Langemann
	mailto:						Manfred.Langemann t t-online.de
	Begin of project:			20.06.2008
	Latest version generated:	28.06.2008
	Filename:					PWsaver.c
	Description:    			Main functions
	Adapted from:				C. Petzold, Window Programming, Issue 5

 *************************************************************/
#include <windows.h>
#include <commdlg.h>
#include "resource.h"

#define EDITID   1
#define UNTITLED TEXT ("(unbenannt)")

LRESULT CALLBACK WndProc			(HWND, UINT, WPARAM, LPARAM);
BOOL    CALLBACK AboutDlgProc		(HWND, UINT, WPARAM, LPARAM);
BOOL	CALLBACK Read_PW_DlgProc	(HWND, UINT, WPARAM, LPARAM);
BOOL	CALLBACK Write_PW_DlgProc	(HWND, UINT, WPARAM, LPARAM);
void	DoCaption					(HWND, TCHAR *);
void	OkMessage					(HWND, TCHAR *, TCHAR *);
short	AskAboutSave				(HWND, TCHAR *);
int		ConfirmPrint				(HWND, TCHAR*);
/*
** Funktionen von PWsaverFile.c
*/
void PWsaverFileInitialize (HWND);
BOOL PWsaverFileOpenDlg    (HWND, PTSTR, PTSTR);
BOOL PWsaverFileSaveDlg    (HWND, PTSTR, PTSTR);
BOOL PWsaverFileRead       (HWND, PTSTR, TCHAR *);
BOOL PWsaverFileWrite      (HWND, PTSTR, TCHAR *);
/*
** Funktionen von PWsaverFind.c
*/
HWND PWsaverFindFindDlg     (HWND);
HWND PWsaverFindReplaceDlg  (HWND);
BOOL PWsaverFindFindText    (HWND, int *, LPFINDREPLACE);
BOOL PWsaverFindReplaceText (HWND, int *, LPFINDREPLACE);
BOOL PWsaverFindNextText    (HWND, int *);
BOOL PWsaverFindValidFind   (void);
/*
** Funktionen von PWsaverFont.c
*/
void PWsaverFontInitialize   (HWND);
BOOL PWsaverFontChooseFont   (HWND);
void PWsaverFontSetFont      (HWND);
void PWsaverFontDeinitialize (void);
/*
** Funktionen von PWsaverPrnt.c
*/
BOOL PWsaverPrntPrintFile (HINSTANCE, HWND, HWND, PTSTR);
/*
** Globale Variablen
*/
static HWND  hDlgModeless;
static TCHAR szAppName[] = TEXT ("PWsaver");
static TCHAR UserPW1[50] = TEXT ("\0");
static TCHAR UserPW2[50] = TEXT ("\0");
static int OpenCommandLine = FALSE;
static TCHAR szFileName[MAX_PATH];
/*************************************************************

	Function WinMain

	Purpose: Main windows function

 *************************************************************/
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
	{
	int			i;
	MSG       msg;
	HWND      hwnd;
	HACCEL    hAccel;
	WNDCLASS  wndclass;
     
	wndclass.style         = CS_HREDRAW | CS_VREDRAW;
	wndclass.lpfnWndProc   = WndProc;
	wndclass.cbClsExtra    = 0;
	wndclass.cbWndExtra    = 0;
	wndclass.hInstance     = hInstance;
	wndclass.hIcon         = LoadIcon (hInstance, TEXT("PWsaver"));
	wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW);
	wndclass.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
	wndclass.lpszMenuName  = szAppName;
	wndclass.lpszClassName = szAppName;
     
	if (!RegisterClass (&wndclass))
		{
/*
** UNICODE-Compilierung ist die einzige realistische Fehlermglichkeit.
** UNICODE ist aber ausgeschaltet, siehe file PWsaverFile.c.
*/
		MessageBox (NULL, TEXT ("Programm arbeitet mit Unicode und setzt Windows NT voraus!"), 
					szAppName, MB_ICONERROR);
		return 0;
		}
/*
** If command line is not empty, then open the file
*/
	if (szCmdLine != NULL && strlen(szCmdLine) != 0 )
		{
/*
** Patch: Remove any " by filtering the command line
*/
		char sz[MAX_PATH];
		char* p = szCmdLine;
		strncpy (sz, szCmdLine, sizeof(sz));
		for (i=0;i<MAX_PATH;++i)
			{
			char c = sz[i];
			if( c == 0 )
				{
				*p = 0;
				break;
				}
			else if( c != '"' )
				{
				*p++ = c;
				}
			}
/*
** End of patch.
*/
		strcpy (szFileName, szCmdLine);
		OpenCommandLine = TRUE;
		}     
/*
** Create the main window and show it
*/
	hwnd = CreateWindow (szAppName, NULL,
							WS_OVERLAPPEDWINDOW,
							CW_USEDEFAULT, CW_USEDEFAULT,
							CW_USEDEFAULT, CW_USEDEFAULT,
							NULL, NULL, hInstance, szCmdLine);
     
	ShowWindow (hwnd, iCmdShow);
	UpdateWindow (hwnd);
	hAccel = LoadAccelerators (hInstance, TEXT ("PW_SAVER"));
/*
** Windows loop
*/
	while (GetMessage (&msg, NULL, 0, 0))
		{
		if (hDlgModeless == NULL || !IsDialogMessage (hDlgModeless, &msg))
			{
			if (!TranslateAccelerator (hwnd, hAccel, &msg))
				{
				TranslateMessage (&msg);
				DispatchMessage (&msg);
				}
			}
		}
	return msg.wParam;
	}
/*************************************************************

	Function WndProc

	Purpose: Window Procedure

 *************************************************************/
LRESULT CALLBACK WndProc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
	{
	static BOOL			bNeedSave = FALSE;
	static HINSTANCE	hInst;
	static HWND			hwndEdit;
	static int			iOffset;
	static TCHAR		szTitleName[MAX_PATH];
	static UINT			messageFindReplace;
	int					iSelBeg, iSelEnd, iEnable;
	static	HFONT		hFont;
	static	LOGFONT		logfont;
	LPFINDREPLACE		pfr;
     
	switch (message)
		{
		case WM_CREATE:
			hInst = ((LPCREATESTRUCT) lParam) -> hInstance;
/*
** Editfenster anlegen
*/
			hwndEdit = CreateWindow (TEXT ("edit"), NULL,
							WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL |
							WS_BORDER | ES_LEFT | ES_MULTILINE |
							ES_NOHIDESEL | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
							0, 0, 0, 0,
							hwnd, (HMENU) EDITID, hInst, NULL);
          
/*
** Maximal 32000 Zeichen
*/
			SendMessage (hwndEdit, EM_LIMITTEXT, 32000, 0L);
//          SendMessage (hwndEdit, EM_LIMITTEXT, 0, 0L);	// 0 = unbegrenzte Textgre
/*
** Initialisierung der Dialogfelder (COMMDLG)
*/
			PWsaverFileInitialize (hwnd);
			PWsaverFontInitialize (hwndEdit);
			messageFindReplace = RegisterWindowMessage (FINDMSGSTRING);
          
			DoCaption (hwnd, szTitleName);
/*
** If command line open file, then open the file
*/
			if (OpenCommandLine == TRUE)
				{
				if (DialogBox (hInst, TEXT ("Read_pw_dlgbox"), hwnd, Read_PW_DlgProc))
					{
/*
** File ffnen, inklusive decryption der daten
*/
					if (PWsaverFileRead (hwndEdit, szFileName, UserPW1))
						{
//						DoCaption (hwnd, szTitleName);
						DoCaption (hwnd, szFileName);
						bNeedSave = FALSE;
						}
					else
						{
						OkMessage (hwnd, TEXT ("Datei %s nicht lesbar!"), szTitleName);
						szFileName[0]  = '\0';
						szTitleName[0] = '\0';
						}
					}
				else
					{
					DestroyWindow (hwnd);
					}
				}
			return 0;

		case WM_SETFOCUS:
			SetFocus (hwndEdit);
			return 0;
          
		case WM_SIZE: 
			MoveWindow (hwndEdit, 0, 0, LOWORD (lParam), HIWORD (lParam), TRUE);
			return 0;
          
		case WM_INITMENUPOPUP:
			switch (lParam)
				{
				case 1:
/*
** Men "Bearbeiten"
*/
/*
** Rckgngig aktivieren, falls das Editfenster das zult
*/
					EnableMenuItem ((HMENU) wParam, IDM_EDIT_UNDO,
                    SendMessage (hwndEdit, EM_CANUNDO, 0, 0L) ? MF_ENABLED : MF_GRAYED);
/*
** Einfgen aktivieren, falls Text in der Zwischenablage
*/
					EnableMenuItem ((HMENU) wParam, IDM_EDIT_PASTE,
					IsClipboardFormatAvailable (CF_TEXT) ? MF_ENABLED : MF_GRAYED);
/*
** Kopieren, Ausschneiden und Lschen aktivieren, falls Text ausgewhlt ist
*/
					SendMessage (hwndEdit, EM_GETSEL, (WPARAM) &iSelBeg, (LPARAM) &iSelEnd);
					iEnable = iSelBeg != iSelEnd ? MF_ENABLED : MF_GRAYED;
               
					EnableMenuItem ((HMENU) wParam, IDM_EDIT_CUT,   iEnable);
					EnableMenuItem ((HMENU) wParam, IDM_EDIT_COPY,  iEnable);
					EnableMenuItem ((HMENU) wParam, IDM_EDIT_CLEAR, iEnable);
					break;
/*
** Men "Suchen"
*/
				case 2:
/*
** Suchen, Ersetzen und Weitersuchen aktivieren,
** falls Dialoge nicht bereits aktiv
*/
					iEnable = hDlgModeless == NULL ? MF_ENABLED : MF_GRAYED;
					EnableMenuItem ((HMENU) wParam, IDM_SEARCH_FIND,    iEnable);
					EnableMenuItem ((HMENU) wParam, IDM_SEARCH_NEXT,    iEnable);
					EnableMenuItem ((HMENU) wParam, IDM_SEARCH_REPLACE, iEnable);
					break;
				}
			return 0;
     
		case WM_COMMAND:
			if (lParam && LOWORD (wParam) == EDITID)
				{
/*
** Nachrichten des Editfensters
*/
				switch (HIWORD (wParam))
					{
					case EN_UPDATE:
						bNeedSave = TRUE;
						return 0;
                    
					case EN_ERRSPACE:
					case EN_MAXTEXT:
						MessageBox (hwnd, TEXT ("Kapazittsgrenze erreicht."),
                                szAppName, MB_OK | MB_ICONSTOP);
						return 0;
					}
				break;
				}
          
			switch (LOWORD (wParam))
				{
/*
** Wahlpunkte des Dateimens
*/
				case IDM_FILE_NEW:
					if (bNeedSave && IDCANCEL == AskAboutSave (hwnd, szTitleName)) return 0;
               
					SetWindowText (hwndEdit, TEXT ("\0"));
					szFileName[0]  = '\0';
					szTitleName[0] = '\0';
					DoCaption (hwnd, szTitleName);
					bNeedSave = FALSE;
					return 0;
               
				case IDM_FILE_OPEN:
					if (bNeedSave && IDCANCEL == AskAboutSave (hwnd, szTitleName)) return 0;
/*
** File Open Dialog ffnen und Filename auswhlen
*/
					if (PWsaverFileOpenDlg (hwnd, szFileName, szTitleName))
						{
/*
** Wenn Filename ausgewhlt, dann Password abfragen
*/
						if (DialogBox (hInst, TEXT ("Read_pw_dlgbox"), hwnd, Read_PW_DlgProc))
							{
/*
** File ffnen, inklusive decryption der daten
*/
							if (PWsaverFileRead (hwndEdit, szFileName, UserPW1))
								{
								DoCaption (hwnd, szTitleName);
								bNeedSave = FALSE;
								}
							else
								{
								OkMessage (hwnd, TEXT ("Datei %s nicht lesbar!"), szTitleName);
								szFileName[0]  = '\0';
								szTitleName[0] = '\0';
								}
							}
						}
					return 0;
               
				case IDM_FILE_SAVE:
/*
** Wenn filename bekannt, dann file schreiben (und encrypten)
*/
					if (szFileName[0])
						{
						if (PWsaverFileWrite (hwndEdit, szFileName, UserPW1))
							{
							bNeedSave = FALSE;
							return 1;
							}
						else
							{
							OkMessage (hwnd, TEXT ("Fehler beim Schreiben von %s!"), szTitleName);
							return 0;
							}
						 }
/*
** Wenn Filename noch nicht bekannt, dann hier weiter mit "Speichern unter"
*/
				case IDM_FILE_SAVE_AS:
					if (PWsaverFileSaveDlg (hwnd, szFileName, szTitleName))
						{
						DoCaption (hwnd, szTitleName);
						if (DialogBox (hInst, TEXT ("Write_pw_dlgbox"), hwnd, Write_PW_DlgProc))
							{                    
							if (PWsaverFileWrite (hwndEdit, szFileName, UserPW1))
								{
								bNeedSave = FALSE;
								return 1;
								}
							else
								{
								OkMessage (hwnd, TEXT ("Fehler beim Schreiben von %s!"), szTitleName);
								return 0;
								}
							}
						}
					return 0;

				case IDM_FILE_PRINT:
					if (ConfirmPrint (hwnd, szFileName))
						{
						if (!PWsaverPrntPrintFile (hInst, hwnd, hwndEdit, szTitleName))
							OkMessage (hwnd, TEXT ("Fehler beim Drucken von %s!"), szTitleName);
						}
					return 0;
               
				case IDM_APP_EXIT:
					SendMessage (hwnd, WM_CLOSE, 0, 0);
					return 0;
/*
** Men "Bearbeiten"
*/
				case IDM_EDIT_UNDO:
					SendMessage (hwndEdit, WM_UNDO, 0, 0);
					return 0;
               
				case IDM_EDIT_CUT:
					SendMessage (hwndEdit, WM_CUT, 0, 0);
					return 0;
               
				case IDM_EDIT_COPY:
					SendMessage (hwndEdit, WM_COPY, 0, 0);
					return 0;
               
				case IDM_EDIT_PASTE:
					SendMessage (hwndEdit, WM_PASTE, 0, 0);
					return 0;
               
				case IDM_EDIT_CLEAR:
					SendMessage (hwndEdit, WM_CLEAR, 0, 0);
					return 0;
               
				case IDM_EDIT_SELECT_ALL:
					SendMessage (hwndEdit, EM_SETSEL, 0, -1);
					return 0;
/*
** Men "Suchen"
*/
				case IDM_SEARCH_FIND:
					SendMessage (hwndEdit, EM_GETSEL, 0, (LPARAM) &iOffset);
					hDlgModeless = PWsaverFindFindDlg (hwnd);
					return 0;
               
				case IDM_SEARCH_NEXT:
					SendMessage (hwndEdit, EM_GETSEL, 0, (LPARAM) &iOffset);
               
					if (PWsaverFindValidFind ())
						{
						PWsaverFindNextText (hwndEdit, &iOffset);
						}
					else
						{
						hDlgModeless = PWsaverFindFindDlg (hwnd);
						}
					return 0;
               
				case IDM_SEARCH_REPLACE:
					SendMessage (hwndEdit, EM_GETSEL, 0, (LPARAM) &iOffset);
					hDlgModeless = PWsaverFindReplaceDlg (hwnd);
					return 0;
               
				case IDM_FORMAT_FONT:
					if (PWsaverFontChooseFont (hwnd)) PWsaverFontSetFont (hwndEdit);
					return 0;
/*
** Men Hilfestellung
*/
				case IDM_HELP:
					OkMessage (hwnd, TEXT ("Siehe Datei PWsaver_Hilfe.txt"), TEXT ("\0"));
					return 0;
               
				case IDM_APP_ABOUT:

					DialogBox (hInst, TEXT ("AboutBox"), hwnd, AboutDlgProc);
					return 0;
				}
			break;
               
		case WM_CLOSE:
			if (!bNeedSave || IDCANCEL != AskAboutSave (hwnd, szTitleName))
			DestroyWindow (hwnd);
			return 0;

		case WM_QUERYENDSESSION :
			if (!bNeedSave || IDCANCEL != AskAboutSave (hwnd, szTitleName)) return 1;
			return 0;
          
		case WM_DESTROY:
			PWsaverFontDeinitialize ();
			PostQuitMessage (0);
			return 0;
          
		default:
			if (message == messageFindReplace)
				{
/*
** Nachrichten fr "Suchen und Ersetzen"
*/
				pfr = (LPFINDREPLACE) lParam;
               
				if (pfr->Flags & FR_DIALOGTERM) hDlgModeless = NULL;
               
				if (pfr->Flags & FR_FINDNEXT)
					{
					if (!PWsaverFindFindText (hwndEdit, &iOffset, pfr))
                         OkMessage (hwnd, TEXT ("Text nicht gefunden!"), TEXT ("\0"));
					}
                    
				if (pfr->Flags & FR_REPLACE || pfr->Flags & FR_REPLACEALL)
					{
					if (!PWsaverFindReplaceText (hwndEdit, &iOffset, pfr))
                         OkMessage (hwnd, TEXT ("Text nicht gefunden!"), TEXT ("\0"));
					}
                         
				if (pfr->Flags & FR_REPLACEALL)
					{
					while (PWsaverFindReplaceText (hwndEdit, &iOffset, pfr));
					}
               return 0;
				}
			break;
			}
	return DefWindowProc (hwnd, message, wParam, lParam);
	}
/*************************************************************

	Function DoCaption

	Purpose: Set Caption of window

 *************************************************************/
void DoCaption (HWND hwnd, TCHAR * szTitleName)
	{
	TCHAR szCaption[64 + MAX_PATH];
     
	wsprintf (szCaption, TEXT ("%s - %s"), szAppName,
				szTitleName[0] ? szTitleName : UNTITLED);
     
	SetWindowText (hwnd, szCaption);
	}
/*************************************************************

	Function OkMessage

	Purpose: Okay mssage window

 *************************************************************/
void OkMessage (HWND hwnd, TCHAR * szMessage, TCHAR * szTitleName)
	{
	TCHAR szBuffer[64 + MAX_PATH];
     
	wsprintf (szBuffer, szMessage, szTitleName[0] ? szTitleName : UNTITLED);
	MessageBox (hwnd, szBuffer, szAppName, MB_OK | MB_ICONEXCLAMATION);
	}
/*************************************************************

	Function AskAboutSave

	Purpose: Ask about to save, if file not saved

 *************************************************************/
short AskAboutSave (HWND hwnd, TCHAR * szTitleName)
	{
	TCHAR szBuffer[64 + MAX_PATH];
	int   iReturn;
     
	wsprintf (szBuffer, TEXT ("Vernderungen in %s speichern?"),
				szTitleName[0] ? szTitleName : UNTITLED);
     
	iReturn = MessageBox (hwnd, szBuffer, szAppName,
							MB_YESNOCANCEL | MB_ICONQUESTION);
	if (iReturn == IDYES)
		{
		if (!SendMessage (hwnd, WM_COMMAND, IDM_FILE_SAVE, 0)) iReturn = IDCANCEL;
		}
	return iReturn;
	}
/*************************************************************

	Function ConfirmPrint

	Purpose: Confirm to print the file

 *************************************************************/
int ConfirmPrint (HWND hwnd, TCHAR * szTitleName)
	{
	TCHAR szBuffer[64 + MAX_PATH];
	int   iReturn;
     
	wsprintf (szBuffer, TEXT ("Soll die Datei:\n\n%s\n\nwirklich ausgedruckt werden ?"),
				szTitleName[0] ? szTitleName : UNTITLED);
     
	iReturn = MessageBox (hwnd, szBuffer, szAppName,
							MB_YESNO | MB_ICONQUESTION);

	if (iReturn == IDYES) return TRUE;

	return FALSE;
	}
/*************************************************************

	Function AboutDlgProc

	Purpose: Info Dialog Procedure

 *************************************************************/
BOOL CALLBACK AboutDlgProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	switch (message)
		{
		case WM_INITDIALOG:
			return TRUE;
          
		case WM_COMMAND:
			switch (LOWORD (wParam))
				{
				case IDOK:
					EndDialog (hDlg, 0);
					return TRUE;
				}
			break;
		}
	return FALSE;
	}
/*************************************************************

	Function Read_PW_DlgProc

	Purpose: Get Read Password dialog procedure

 *************************************************************/
BOOL CALLBACK Read_PW_DlgProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	UINT		len;
	UINT		i;
	TCHAR		*pC;
	int			iValid = TRUE;
	i=1;

	switch (message)
		{
		case WM_INITDIALOG:
/*
** Set focus to edit field
*/
			SetFocus (GetDlgItem (hDlg, IDC_GETPW));
// SetDlgItemText (hDlg, IDC_GETPW, TEXT("\0"));
// SendMessage(hDlg, WM_NEXTDLGCTL, (WPARAM)GetDlgItem (hDlg, IDC_GETPW), TRUE);
/*
** Return with FALSE to set the focus to the edit field as specified above
*/
			return FALSE;
          
		case WM_COMMAND:
			switch (LOWORD (wParam))
				{
				case IDOK:
/*
** Get the password1 from the edit field and save it in UserPW
*/
					len = GetDlgItemText (hDlg, IDC_GETPW, UserPW1, 50);
/*
** If password length not correct, the error message
*/
					if (len < 6 || len > 40)
						{
						OkMessage (hDlg, TEXT ("Passwort muss mindestens 6 und max. 40 Zeichen haben !\nGltige Zeichen: A-Z, a-z, 0-9, keine Umlaute"), TEXT ("\0"));
						SetDlgItemText (hDlg, IDC_GETPW, TEXT("\0"));
						SetFocus (GetDlgItem (hDlg, IDC_GETPW));
						return TRUE;
						}
/*
** Check on correct characters of password
*/
					for (i=0;i<len;i++)
						{
						pC = &UserPW1[i];
						if (!((*pC>='0' && *pC<='9') || (*pC>='a' && *pC<='z') || (*pC>='A' && *pC<='Z'))) iValid = FALSE;
						}
					if (iValid == FALSE)
						{
						OkMessage (hDlg, TEXT ("Passwort enthlt ungltige Zeichen !\nGltige Zeichen: 0-9, A-Z, a-z, Keine Umlaute"), TEXT ("\0"));
						SetDlgItemText (hDlg, IDC_GETPW, TEXT("\0"));
						SetFocus (GetDlgItem (hDlg, IDC_GETPW));
						return TRUE;
						}
/*
** Wenn alles korrekt, dann Dialog schlieen mit TRUE
*/
					EndDialog (hDlg, TRUE);
					return TRUE;
/*
** Wenn Cancel, dann Dialog schlieen mit FALSE
*/
				case IDCANCEL:
					EndDialog (hDlg, FALSE);
					return TRUE;
				}
			break;
		}
	return FALSE;
	}

/*************************************************************

	Function Write_PW_DlgProc

	Purpose: Get Write Password dialog procedure

 *************************************************************/
BOOL CALLBACK Write_PW_DlgProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
	{
	UINT		len;
	UINT		i;
	TCHAR		*pC;
	int			iValid = TRUE;
	i=1;

	switch (message)
		{
		case WM_INITDIALOG:
/*
** Set focus to edit field
*/
			SetFocus (GetDlgItem (hDlg, IDC_GETPW1));
// SetDlgItemText (hDlg, IDC_GETPW, TEXT("\0"));
// SendMessage(hDlg, WM_NEXTDLGCTL, (WPARAM)GetDlgItem (hDlg, IDC_GETPW), TRUE);
/*
** Return with FALSE to set the focus to the edit field as specified above
*/
			return FALSE;
          
		case WM_COMMAND:
			switch (LOWORD (wParam))
				{
				case IDOK:
/*
** Get the password1 from the edit field and save it in UserPW1
*/
					len = GetDlgItemText (hDlg, IDC_GETPW1, UserPW1, 50);
/*
** If password length not correct, the error message
*/
					if (len < 6 || len > 40)
						{
						OkMessage (hDlg, TEXT ("Passwort muss mindestens 6 und max. 40 Zeichen haben !\nGltige Zeichen: A-Z, a-z, 0-9, keine Umlaute"), TEXT ("\0"));
						SetDlgItemText (hDlg, IDC_GETPW1, TEXT("\0"));
						SetDlgItemText (hDlg, IDC_GETPW2, TEXT("\0"));
						SetFocus (GetDlgItem (hDlg, IDC_GETPW1));
						return TRUE;
						}
/*
** Get the password2 from the edit field and compare it with UserPW2
*/
					len = GetDlgItemText (hDlg, IDC_GETPW2, UserPW2, 50);
					if (strcmp (UserPW1, UserPW2))
						{
						OkMessage (hDlg, TEXT ("Erstes Passwort stimmt nicht mit zweitem berein !"), TEXT ("\0"));
						SetDlgItemText (hDlg, IDC_GETPW1, TEXT("\0"));
						SetDlgItemText (hDlg, IDC_GETPW2, TEXT("\0"));
						SetFocus (GetDlgItem (hDlg, IDC_GETPW1));
						return TRUE;
						}
/*
** Check on correct characters of password
*/
					for (i=0;i<len;i++)
						{
						pC = &UserPW1[i];
						if (!((*pC>='0' && *pC<='9') || (*pC>='a' && *pC<='z') || (*pC>='A' && *pC<='Z'))) iValid = FALSE;
						}
					if (iValid == FALSE)
						{
						OkMessage (hDlg, TEXT ("Passwort enthlt ungltige Zeichen !\nGltige Zeichen: A-Z, a-z, 0-9, keine Umlaute"), TEXT ("\0"));
						SetDlgItemText (hDlg, IDC_GETPW1, TEXT("\0"));
						SetDlgItemText (hDlg, IDC_GETPW2, TEXT("\0"));
						SetFocus (GetDlgItem (hDlg, IDC_GETPW1));
						return TRUE;
						}
/*
** Wenn alles korrekt, dann Dialog schlieen mit TRUE
*/
					EndDialog (hDlg, TRUE);
					return TRUE;
/*
** Wenn Cancel, dann Dialog schlieen mit FALSE
*/
				case IDCANCEL:
					EndDialog (hDlg, FALSE);
					return TRUE;
				}
			break;
		}
	return FALSE;
	}
