/************************************************************

	Author:						Manfred Langemann
	mailto:						Manfred.Langemann t t-online.de
	Begin of project:			20.06.2008
	Latest version generated:	28.06.2008
	Filename:					PWsaverFile.c
	Description:    			Main routine for File handling
	Adapted from:				C. Petzold, Window Programming, Issue 5

	Note: The definition UNICODE has been removed from the project
	settings in	Project-->Settings-->C/C++ in field Preprocessor
	Definitions with {.....,UNICODE, _UNICODE}

 *************************************************************/
#include <windows.h>
#include <commdlg.h>
#include "Blowfish.h"
/*
** Static global variable
*/
static OPENFILENAME ofn;
/*************************************************************

	Function PWsaverFileInitialize

	Purpose: Initialise the OpenFileName structure

 *************************************************************/
void PWsaverFileInitialize (HWND hwnd)
	{
//     static TCHAR szFilter[] = TEXT ("Textdateien (*.TXT)\0*.txt\0")  \
//                               TEXT ("ASCII-Dateien (*.ASC)\0*.asc\0") \
//                               TEXT ("Alle Dateien (*.*)\0*.*\0\0");
	static TCHAR szFilter[] =  TEXT ("Passwort Dateien (*.PWD)\0*.pwd\0\0");
/*
** Definition of OFN structure
*/     
	ofn.lStructSize       = sizeof (OPENFILENAME);
	ofn.hwndOwner         = hwnd;
	ofn.hInstance         = NULL;
	ofn.lpstrFilter       = szFilter;
	ofn.lpstrCustomFilter = NULL;
	ofn.nMaxCustFilter    = 0;
	ofn.nFilterIndex      = 0;
	ofn.lpstrFile         = NULL;          // Bei ffnen bzw. Schlieen gesetzt
	ofn.nMaxFile          = MAX_PATH;
	ofn.lpstrFileTitle    = NULL;          // Bei ffnen bzw. Schlieen gesetzt
	ofn.nMaxFileTitle     = MAX_PATH;
	ofn.lpstrInitialDir   = NULL;
	ofn.lpstrTitle        = NULL;
	ofn.Flags             = 0;             // Bei ffnen bzw. Schlieen gesetzt
	ofn.nFileOffset       = 0;
	ofn.nFileExtension    = 0;
//	ofn.lpstrDefExt       = TEXT ("txt");
	ofn.lpstrDefExt       = TEXT ("pwd");
	ofn.lCustData         = 0L;
	ofn.lpfnHook          = NULL;
	ofn.lpTemplateName    = NULL;
	}
/*************************************************************

	Function PWsaverFileOpenDlg

	Purpose: Open File Dialog

 *************************************************************/
BOOL PWsaverFileOpenDlg (HWND hwnd, PTSTR pstrFileName, PTSTR pstrTitleName)
	{
	ofn.hwndOwner         = hwnd;
	ofn.lpstrFile         = pstrFileName;
	ofn.lpstrFileTitle    = pstrTitleName;
	ofn.Flags             = OFN_HIDEREADONLY | OFN_CREATEPROMPT;
     
	return GetOpenFileName (&ofn);
	}
/*************************************************************

	Function PWsaverFileSaveDlg

	Purpose: Save File Dialog

 *************************************************************/
BOOL PWsaverFileSaveDlg (HWND hwnd, PTSTR pstrFileName, PTSTR pstrTitleName)
	{
	ofn.hwndOwner         = hwnd;
	ofn.lpstrFile         = pstrFileName;
	ofn.lpstrFileTitle    = pstrTitleName;
	ofn.Flags             = OFN_OVERWRITEPROMPT;
     
	return GetSaveFileName (&ofn);
	}
/*************************************************************

	Function PWsaverFileRead

	Purpose: Read File Data, decrypt data and write to text window.
	Note: UNICODE version has been removed, see begin of file above.

 *************************************************************/
BOOL PWsaverFileRead (HWND hwndEdit, PTSTR pstrFileName, TCHAR* UserPW)
	{
	BYTE   bySwap;
	DWORD  dwBytesRead;
	HANDLE hFile;
	int    i, iFileLength, iUniTest;
	PBYTE  pBuffer, pText, pConv;
	PBYTE  pText_i, pConv_i;
/*
** Datei ffnen
*/
	if (INVALID_HANDLE_VALUE == 
			(hFile = CreateFile (pstrFileName, GENERIC_READ, FILE_SHARE_READ,
					NULL, OPEN_EXISTING, 0, NULL))) return FALSE;
/*
** Dateigre in Bytes ermitteln, zwei Bytes fr ein abschlieendes NUL 
** dazuaddieren (Unicode!), Speicher belegen
*/
	iFileLength = GetFileSize (hFile, NULL);
	pBuffer = malloc (iFileLength + 2);
/*
** Dateiinhalt lesen und zwei NUL-Zeichen anhngen
*/
	ReadFile (hFile, pBuffer, iFileLength, &dwBytesRead, NULL);
	CloseHandle (hFile);
	pBuffer[iFileLength] = '\0';
	pBuffer[iFileLength + 1] = '\0';
/*
** Unicode-Text?
*/
	iUniTest = IS_TEXT_UNICODE_SIGNATURE | IS_TEXT_UNICODE_REVERSE_SIGNATURE;

	if (IsTextUnicode (pBuffer, iFileLength, &iUniTest))
		{
		pText = pBuffer + 2;
		iFileLength -= 2;

		if (iUniTest & IS_TEXT_UNICODE_REVERSE_SIGNATURE)
			{
			for (i=0;i<iFileLength/2;i++)
				{
				bySwap = ((BYTE *) pText) [2 * i];
				((BYTE *) pText) [2 * i] = ((BYTE *) pText) [2 * i + 1];
				((BYTE *) pText) [2 * i + 1] = bySwap;
				}
			}
/*
** Speicher fr eine eventuell notwendige Konvertierung belegen
*/
		pConv = malloc (iFileLength + 2);
/*
**	Wenn das Editfenster nicht mit Unicode zurechtkommt, Konvertierung
*/
#ifndef UNICODE
		WideCharToMultiByte (CP_ACP, 0, (PWSTR) pText, -1, pConv, 
							iFileLength + 2, NULL, NULL);
/*
**	Wenn das Editfenster mit Unicode zurechtkommt, einfache Kopie
*/
#else
		lstrcpy ((PTSTR) pConv, (PTSTR) pText);
#endif
		}
	else      // ANSI-Text
		{
		pText = pBuffer;
/*
**	Speicher fr eine eventuell notwendige Konvertierung belegen
*/
		pConv = malloc (2 * iFileLength + 2);
/*
** Wenn das Editfenster mit Unicode arbeitet, Konvertierung
*/
#ifdef UNICODE
		MultiByteToWideChar (CP_ACP, 0, pText, -1, (PTSTR) pConv, 
							iFileLength + 1);
/*
** ANSI-Text, ANSI-Fenster: reine Kopie
** Die function "lstrcpy ((PTSTR) pConv, (PTSTR) pText);" geht hier nicht,
** weil sie bei einem Wert von '\0' aufhrt zu kopieren (weil codierter Text).
*/
#else
		*(pText+iFileLength+1) = '\0';
		pConv_i = pConv;
		pText_i = pText;
		for (i=0;i<(iFileLength+1);i++)
			{
			*(pConv_i++) = *(pText_i++);
			}
#endif
		}
/*
** Now decrypt text in buffer
*/
	DoBlowfish (pConv, iFileLength, DECRYPT_BLOWFISH, UserPW);
/*
** Put decrypted text to edit window
*/
	SetWindowText (hwndEdit, (PTSTR) pConv);
	free (pBuffer);
	free (pConv);
   
	return TRUE;
	}
/*************************************************************

	Function PWsaverFileRead

	Purpose: Encrypt data in text window and write data to file.
	Note: UNICODE version has been removed, see begin of file above.

 *************************************************************/
BOOL PWsaverFileWrite (HWND hwndEdit, PTSTR pstrFileName, TCHAR* UserPW)
	{
	DWORD	dwBytesWritten;
	HANDLE	hFile;
	int		iLength, k;
	PTSTR	pstrBuffer;
	WORD	wByteOrderMark = 0xFEFF;
	char	cAppend[9] = {"        "};
/*
** Datei ffnen bzw. neu anlegen, falls noch nicht vorhanden
*/
     if (INVALID_HANDLE_VALUE == 
               (hFile = CreateFile (pstrFileName, GENERIC_WRITE, 0, 
                                    NULL, CREATE_ALWAYS, 0, NULL))) return FALSE;

/*
** Zahl der Zeichen (nicht unbedingt Bytes!) im Editfenster ermitteln
** und einen Puffer dafr belegen
*/
	iLength = GetWindowTextLength (hwndEdit);
/*
** Correct length of text in edit window to a multiple of 8 bytes
*/
	k = iLength/8;
	k = 8-(iLength-k*8);
	if (k==8) k=0;
	if (k!=0)
		{
		cAppend[k] ='\0';
		SendMessage (hwndEdit, EM_SETSEL, (WPARAM)iLength, (LPARAM)iLength);
		SendMessage (hwndEdit, EM_REPLACESEL, (WPARAM)FALSE, (LPARAM)cAppend);
		iLength = GetWindowTextLength(hwndEdit);
		SendMessage (hwndEdit, EM_SETSEL, (WPARAM)iLength, (LPARAM)iLength);
		}
/*
** Allocate memory for buffer
*/
	pstrBuffer = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR));

	if (!pstrBuffer)
		{
		CloseHandle (hFile);
		return FALSE;
		}

/*
** Wenn das Editfenster mit Unicode arbeitet: Marker fr die Byte-
** Reihenfolge als erstes in die Datei schreiben
*/
#ifdef UNICODE
	WriteFile (hFile, &wByteOrderMark, 2, &dwBytesWritten, NULL);
#endif
/*
** Text aus dem Editfenster in den Puffer kopieren
*/
	GetWindowText (hwndEdit, pstrBuffer, iLength + 1);
/*
** Now encrypt text in buffer
*/
	DoBlowfish (pstrBuffer, iLength, ENCRYPT_BLOWFISH, UserPW);
/*
** Now write test in buffer to file
*/
	WriteFile (hFile, pstrBuffer, iLength * sizeof (TCHAR), 
				&dwBytesWritten, NULL);

	if ((iLength * sizeof (TCHAR)) != (int) dwBytesWritten)
		{  // Schreibfehler - nicht alle Bytes geschrieben
		CloseHandle (hFile);
		free (pstrBuffer);
		return FALSE;
		}
     
	CloseHandle (hFile);
	free (pstrBuffer);
 
	return TRUE;
	}
