/************************************************************

	Author:						Manfred Langemann
	mailto:						Manfred.Langemann t t-online.de
	Begin of project:			20.06.2008
	Latest version generated:	28.06.2008
	Filename:					PWsaverFind.c
	Description:    			Find functions
	Adapted from:				C. Petzold, Window Programming, Issue 5

 *************************************************************/
#include <windows.h>
#include <commdlg.h>
#include <tchar.h>       // tcsstr (gemeinsame strstr-Variante fr ANSI und Unicode)

#define MAX_STRING_LEN   256

static TCHAR szFindText [MAX_STRING_LEN];
static TCHAR szReplText [MAX_STRING_LEN];
/*************************************************************

	Function PWsaverFindFindDlg

	Purpose: Find dialog

 *************************************************************/
HWND PWsaverFindFindDlg (HWND hwnd)
	{
	static FINDREPLACE fr;       // // fr moduslosen Dialog, *mu* statisch sein!

	fr.lStructSize      = sizeof (FINDREPLACE);
	fr.hwndOwner        = hwnd;
	fr.hInstance        = NULL;
	fr.Flags            = FR_HIDEUPDOWN | FR_HIDEMATCHCASE | FR_HIDEWHOLEWORD;
	fr.lpstrFindWhat    = szFindText;
	fr.lpstrReplaceWith = NULL;
	fr.wFindWhatLen     = MAX_STRING_LEN;
	fr.wReplaceWithLen  = 0;
	fr.lCustData        = 0;
	fr.lpfnHook         = NULL;
	fr.lpTemplateName   = NULL;

	return FindText (&fr);
	}
/*************************************************************

	Function PWsaverFindReplaceDlg

	Purpose: Replace dialog function

 *************************************************************/
HWND PWsaverFindReplaceDlg (HWND hwnd)
	{
	static FINDREPLACE fr;       // // fr moduslosen Dialog, *mu* statisch sein!

	fr.lStructSize      = sizeof (FINDREPLACE);
	fr.hwndOwner        = hwnd;
	fr.hInstance        = NULL;
	fr.Flags            = FR_HIDEUPDOWN | FR_HIDEMATCHCASE | FR_HIDEWHOLEWORD;
	fr.lpstrFindWhat    = szFindText;
	fr.lpstrReplaceWith = szReplText;
	fr.wFindWhatLen     = MAX_STRING_LEN;
	fr.wReplaceWithLen  = MAX_STRING_LEN;
	fr.lCustData        = 0;
	fr.lpfnHook         = NULL;
	fr.lpTemplateName   = NULL;
 
	return ReplaceText (&fr);
	}
/*************************************************************

	Function PWsaverFindFindText

	Purpose: Find text function

 *************************************************************/
BOOL PWsaverFindFindText (HWND hwndEdit, int * piSearchOffset, LPFINDREPLACE pfr)
	{
	int    iLength, iPos;
	PTSTR  pstrDoc, pstrPos;
/*
** Dokument in einen Puffer kopieren
*/
	iLength = GetWindowTextLength (hwndEdit);
	if (NULL == (pstrDoc = (PTSTR) malloc ((iLength + 1) * sizeof (TCHAR)))) return FALSE;
	GetWindowText (hwndEdit, pstrDoc, iLength + 1);
/*
** Suche nach dem Text innerhalb des Dokuments
*/
	pstrPos = _tcsstr (pstrDoc + * piSearchOffset, pfr->lpstrFindWhat);
	free (pstrDoc);
/*
** Abbruch mit Fehlercode, falls erfolglos
*/
	if (pstrPos == NULL) return FALSE;
/*
** Sonst Startadresse bestimmen und neu setzen fr nchste Suche
*/
	iPos = pstrPos - pstrDoc;
	*piSearchOffset = iPos + lstrlen (pfr->lpstrFindWhat);
/*
** Auswhlen des gefundenen Texts
*/
	SendMessage (hwndEdit, EM_SETSEL, iPos, * piSearchOffset);
	SendMessage (hwndEdit, EM_SCROLLCARET, 0, 0);
 
	return TRUE;
	}
/*************************************************************

	Function PWsaverFindNextText

	Purpose: Find next text function

 *************************************************************/
BOOL PWsaverFindNextText (HWND hwndEdit, int * piSearchOffset)
	{
	FINDREPLACE fr;

	fr.lpstrFindWhat = szFindText;

	return PWsaverFindFindText (hwndEdit, piSearchOffset, &fr);
	}
/*************************************************************

	Function PWsaverFindReplaceText

	Purpose: Find and replace text function

 *************************************************************/
BOOL PWsaverFindReplaceText (HWND hwndEdit, int * piSearchOffset, LPFINDREPLACE pfr)
	{
/*
** Fundstelle lokalisieren
*/
	if (!PWsaverFindFindText (hwndEdit, piSearchOffset, pfr)) return FALSE;
/*
** ... und ersetzen
*/
	SendMessage (hwndEdit, EM_REPLACESEL, 0, (LPARAM) pfr->lpstrReplaceWith);
 
	return TRUE;
	}
/*************************************************************

	Function PWsaverFindValidFind

	Purpose: Find valid function

 *************************************************************/
BOOL PWsaverFindValidFind (void)
	{
	return * szFindText != '\0';
	}
